
# q-expansion of newform 110.4.a.g, downloaded from the LMFDB on 03 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 110
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [7], [-5], [11], [11], [2], [-9], [-85], [-138], [45], [227], [-19], [-138], [-88], [-534], [297], [-450], [287], [-304], [777], [962], [290], [1422], [-1455], [116], [402], [1412], [2136], [230], [-108], [1376], [-1593], [756], [1460], [-2145], [2702], [1421], [-2773], [-2109], [-3558], [300], [3122], [-1548], [827], [456], [2825], [-2233], [5402], [2826], [-2410], [687], [1650], [2882], [-7698], [-6354], [-5223], [-4500], [-1708], [5636], [-5058], [-4858], [9402], [-6574], [-8253], [-4858], [711], [8552], [1121], [10986], [-1090], [462], [-60], [-1924], [-6238], [830], [3762], [-5550], [8786], [-3333], [-12820], [13680], [4952], [16032], [692], [-17920], [15852], [6930], [-619], [-10533], [-11698], [1431], [-4740], [-2104], [-12933], [17540], [7632], [2340], [-16338], [7712], [14537], [14096], [2706], [20502], [12960], [-14443], [-10234], [9741], [-16518], [13635], [-13438], [-15199], [21122], [-2304], [-14020], [22457], [8847], [-1213], [-9774], [-14433], [22905], [2942], [-13813], [8436], [10377], [18602], [-31353], [27470], [-10425], [-26854], [-22228], [-12640], [24507], [17087], [7106], [-2238], [-12730], [-22323], [40376], [-21114], [-38550], [-43063], [-2298], [-5428], [-5004], [-25180], [-8760], [35462], [-12489], [10970], [36942], [-48094], [-28878], [6497], [15696], [9551], [1317], [43460], [40995], [-31174], [-41103], [18861], [4857], [-18859], [-20568], [45636], [-33618], [-42928], [55316]]
